package sun.misc;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;

public class UUDecoder extends CharacterDecoder
{
    public String bufferName;
    public int mode;
    private byte[] decoderBuffer;
    
    public UUDecoder() {
        this.decoderBuffer = new byte[4];
    }
    
    @Override
    protected int bytesPerAtom() {
        return 3;
    }
    
    @Override
    protected int bytesPerLine() {
        return 45;
    }
    
    @Override
    protected void decodeAtom(PushbackInputStream pushbackInputStream, OutputStream outputStream, int n) throws IOException {
        StringBuffer sb = new StringBuffer();
        int i = 0;
        while (i < 4) {
            int read = pushbackInputStream.read();
            if (read == -1) {
                throw new CEStreamExhausted();
            }
            sb.append((char)read);
            this.decoderBuffer[i] = (byte)(read - 32 & 0x3F);
            ++i;
        }
        int n2 = (this.decoderBuffer[0] << 2 & 0xFC) | (this.decoderBuffer[1] >>> 4 & 0x3);
        int n3 = (this.decoderBuffer[1] << 4 & 0xF0) | (this.decoderBuffer[2] >>> 2 & 0xF);
        int n4 = (this.decoderBuffer[2] << 6 & 0xC0) | (this.decoderBuffer[3] & 0x3F);
        outputStream.write((int)(byte)(n2 & 0xFF));
        if (n > 1) {
            outputStream.write((int)(byte)(n3 & 0xFF));
        }
        if (n > 2) {
            outputStream.write((int)(byte)(n4 & 0xFF));
        }
    }
    
    @Override
    protected void decodeBufferPrefix(PushbackInputStream pushbackInputStream, OutputStream outputStream) throws IOException {
        StringBuffer sb = new StringBuffer(32);
        int n = 1;
        while (true) {
            int n2 = pushbackInputStream.read();
            if (n2 == -1) {
                throw new CEFormatException("UUDecoder: No begin line.");
            }
            if (n2 == 98 && n != 0) {
                n2 = pushbackInputStream.read();
                if (n2 == 101) {
                    while (n2 != 10 && n2 != 13) {
                        n2 = pushbackInputStream.read();
                        if (n2 == -1) {
                            throw new CEFormatException("UUDecoder: No begin line.");
                        }
                        if (n2 == 10 || n2 == 13) {
                            continue;
                        }
                        sb.append((char)n2);
                    }
                    String string = sb.toString();
                    if (string.indexOf(32) != 3) {
                        throw new CEFormatException("UUDecoder: Malformed begin line.");
                    }
                    this.mode = Integer.parseInt(string.substring(4, 7));
                    this.bufferName = string.substring(string.indexOf(32, 6) + 1);
                    if (n2 == 13) {
                        int read = pushbackInputStream.read();
                        if (read != 10 && read != -1) {
                            pushbackInputStream.unread(read);
                        }
                    }
                    return;
                }
            }
            n = (n2 == 10 || n2 == 13);
        }
    }
    
    @Override
    protected int decodeLinePrefix(PushbackInputStream pushbackInputStream, OutputStream outputStream) throws IOException {
        int read = pushbackInputStream.read();
        if (read == 32) {
            pushbackInputStream.read();
            int read2 = pushbackInputStream.read();
            if (read2 != 10 && read2 != -1) {
                pushbackInputStream.unread(read2);
            }
            throw new CEStreamExhausted();
        }
        else {
            if (read == -1) {
                throw new CEFormatException("UUDecoder: Short Buffer.");
            }
            int n = read - 32 & 0x3F;
            if (n > this.bytesPerLine()) {
                throw new CEFormatException("UUDecoder: Bad Line Length.");
            }
            return n;
        }
    }
    
    @Override
    protected void decodeLineSuffix(PushbackInputStream pushbackInputStream, OutputStream outputStream) throws IOException {
        int i;
        do {
            i = pushbackInputStream.read();
            if (i == -1) {
                throw new CEStreamExhausted();
            }
            if (i == 10) {
                return;
            }
            continue;
        } while (i != 13);
        int read = pushbackInputStream.read();
        if (read != 10 && read != -1) {
            pushbackInputStream.unread(read);
            return;
        }
        return;
    }
    
    @Override
    protected void decodeBufferSuffix(PushbackInputStream pushbackInputStream, OutputStream outputStream) throws IOException {
        pushbackInputStream.read(this.decoderBuffer);
        if (this.decoderBuffer[0] != 101 || this.decoderBuffer[1] != 110 || this.decoderBuffer[2] != 100) {
            throw new CEFormatException("UUDecoder: Missing 'end' line.");
        }
        return;
    }
}
